@@ -55,7 +55,7 @@ class AccountController(object):
         self.auto_create_account_prefix = \
             conf.get('auto_create_account_prefix') or '.'
         swift.common.db.DB_PREALLOCATION = \
-            conf.get('db_preallocation', 't').lower() in TRUE_VALUES
+            conf.get('db_preallocation', 'f').lower() in TRUE_VALUES
 
     def _get_account_broker(self, drive, part, account):
         hsh = hash_path(account)
@@ -233,7 +233,7 @@ class AccountController(object):
                 limit = int(given_limit)
                 if limit > ACCOUNT_LISTING_LIMIT:
                     self.logger.increment('GET.errors')
-                    return  HTTPPreconditionFailed(request=req,
+                    return HTTPPreconditionFailed(request=req,
                         body='Maximum limit is %d' % ACCOUNT_LISTING_LIMIT)
             marker = get_param(req, 'marker', '')
             end_marker = get_param(req, 'end_marker')
