@@ -125,6 +125,7 @@ from swift.common.utils import cache_from_env, get_logger, human_readable, \
                                split_path, TRUE_VALUES
 from swift.common.wsgi import make_pre_authed_env, make_pre_authed_request, \
                               WSGIContext
+from swift.common.http import is_success, is_redirection, HTTP_NOT_FOUND
 
 
 def quote(value, safe='/'):
@@ -183,7 +184,7 @@ class _StaticWebContext(WSGIContext):
                 '/%s/%s/%s/%s%s' % (self.version, self.account, self.container,
                                     self._get_status_int(), self._error),
                 self.agent))
-        if self._get_status_int() // 100 == 2:
+        if is_success(self._get_status_int()):
             start_response(save_response_status, self._response_headers,
                            self._response_exc_info)
             return resp
@@ -213,7 +214,7 @@ class _StaticWebContext(WSGIContext):
         resp = make_pre_authed_request(env, 'HEAD',
                 '/%s/%s/%s' % (self.version, self.account, self.container),
                 agent=self.agent).get_response(self.app)
-        if resp.status_int // 100 == 2:
+        if is_success(resp.status_int):
             self._index = \
                 resp.headers.get('x-container-meta-web-index', '').strip()
             self._error = \
@@ -249,7 +250,7 @@ class _StaticWebContext(WSGIContext):
         else:
             prefix = ''
         resp = self._app_call(tmp_env)
-        if self._get_status_int() // 100 != 2:
+        if not is_success(self._get_status_int()):
             return self._error_response(resp, env, start_response)
         listing = None
         body = ''.join(resp)
@@ -363,9 +364,10 @@ class _StaticWebContext(WSGIContext):
         tmp_env['PATH_INFO'] += self._index
         resp = self._app_call(tmp_env)
         status_int = self._get_status_int()
-        if status_int == 404:
+        if status_int == HTTP_NOT_FOUND:
             return self._listing(env, start_response)
-        elif self._get_status_int() // 100 not in (2, 3):
+        elif not is_success(self._get_status_int()) or \
+             not is_redirection(self._get_status_int()):
             return self._error_response(resp, env, start_response)
         start_response(self._response_status, self._response_headers,
                        self._response_exc_info)
@@ -384,16 +386,16 @@ class _StaticWebContext(WSGIContext):
             '%s StaticWeb' % env.get('HTTP_USER_AGENT')
         resp = self._app_call(tmp_env)
         status_int = self._get_status_int()
-        if status_int // 100 in (2, 3):
+        if is_success(status_int) or is_redirection(status_int):
             start_response(self._response_status, self._response_headers,
                            self._response_exc_info)
             return resp
-        if status_int != 404:
+        if status_int != HTTP_NOT_FOUND:
             return self._error_response(resp, env, start_response)
         self._get_container_info(env)
         if not self._listings and not self._index:
             return self.app(env, start_response)
-        status_int = 404
+        status_int = HTTP_NOT_FOUND
         if self._index:
             tmp_env = dict(env)
             tmp_env['HTTP_USER_AGENT'] = \
@@ -403,7 +405,7 @@ class _StaticWebContext(WSGIContext):
             tmp_env['PATH_INFO'] += self._index
             resp = self._app_call(tmp_env)
             status_int = self._get_status_int()
-            if status_int // 100 in (2, 3):
+            if is_success(status_int) or is_redirection(status_int):
                 if env['PATH_INFO'][-1] != '/':
                     resp = HTTPMovedPermanently(
                         location=env['PATH_INFO'] + '/')
@@ -412,7 +414,7 @@ class _StaticWebContext(WSGIContext):
                 start_response(self._response_status, self._response_headers,
                                self._response_exc_info)
                 return resp
-        if status_int == 404:
+        if status_int == HTTP_NOT_FOUND:
             if env['PATH_INFO'][-1] != '/':
                 tmp_env = make_pre_authed_env(env, 'GET',
                             '/%s/%s/%s' % (self.version, self.account,
@@ -422,7 +424,7 @@ class _StaticWebContext(WSGIContext):
                     '=/&limit=1&prefix=%s' % quote(self.obj + '/')
                 resp = self._app_call(tmp_env)
                 body = ''.join(resp)
-                if self._get_status_int() // 100 != 2 or not body or \
+                if not is_success(self._get_status_int()) or not body or \
                         not json.loads(body):
                     resp = HTTPNotFound()(env, self._start_response)
                     return self._error_response(resp, env, start_response)
