@@ -32,8 +32,7 @@ from eventlet.support.greenlets import GreenletExit
 
 from swift.common.ring import Ring
 from swift.common.utils import whataremyips, unlink_older_than, lock_path, \
-        compute_eta, get_logger, write_pickle, renamer, dump_recon_cache, \
-        TRUE_VALUES
+        compute_eta, get_logger, write_pickle, renamer, dump_recon_cache
 from swift.common.bufferedhttp import http_connect
 from swift.common.daemon import Daemon
 from swift.common.http import HTTP_OK, HTTP_INSUFFICIENT_STORAGE
@@ -247,11 +246,9 @@ class ObjectReplicator(Daemon):
         self.rsync_io_timeout = conf.get('rsync_io_timeout', '30')
         self.http_timeout = int(conf.get('http_timeout', 60))
         self.lockup_timeout = int(conf.get('lockup_timeout', 1800))
-        self.recon_enable = conf.get(
-                'recon_enable', 'no').lower() in TRUE_VALUES
-        self.recon_cache_path = conf.get(
-                'recon_cache_path', '/var/cache/swift')
-        self.recon_object = os.path.join(self.recon_cache_path, "object.recon")
+        self.recon_cache_path = conf.get('recon_cache_path',
+                                         '/var/cache/swift')
+        self.rcache = os.path.join(self.recon_cache_path, "object.recon")
 
     def _rsync(self, args):
         """
@@ -358,6 +355,7 @@ class ObjectReplicator(Daemon):
             return [suff for suff in os.listdir(path)
                     if len(suff) == 3 and isdir(join(path, suff))]
         self.replication_count += 1
+        self.logger.increment('partition.delete.count.%s' % (job['device'],))
         begin = time.time()
         try:
             responses = []
@@ -373,13 +371,14 @@ class ObjectReplicator(Daemon):
                           headers={'Content-Length': '0'}).getresponse().read()
                     responses.append(success)
             if not suffixes or (len(responses) == \
-                        self.object_ring.replica_count and all(responses)):
+                        len(job['nodes']) and all(responses)):
                 self.logger.info(_("Removing partition: %s"), job['path'])
                 tpool.execute(shutil.rmtree, job['path'], ignore_errors=True)
         except (Exception, Timeout):
             self.logger.exception(_("Error syncing handoff partition"))
         finally:
             self.partition_times.append(time.time() - begin)
+            self.logger.timing_since('partition.delete.timing', begin)
 
     def update(self, job):
         """
@@ -388,6 +387,7 @@ class ObjectReplicator(Daemon):
         :param job: a dict containing info about the partition to be replicated
         """
         self.replication_count += 1
+        self.logger.increment('partition.update.count.%s' % (job['device'],))
         begin = time.time()
         try:
             hashed, local_hash = tpool.execute(tpooled_get_hashes, job['path'],
@@ -397,7 +397,8 @@ class ObjectReplicator(Daemon):
             if isinstance(hashed, BaseException):
                 raise hashed
             self.suffix_hash += hashed
-            attempts_left = self.object_ring.replica_count - 1
+            self.logger.update_stats('suffix.hashes', hashed)
+            attempts_left = len(job['nodes'])
             nodes = itertools.chain(job['nodes'],
                         self.object_ring.get_more_nodes(int(job['partition'])))
             while attempts_left > 0:
@@ -431,6 +432,7 @@ class ObjectReplicator(Daemon):
                     # See tpooled_get_hashes "Hack".
                     if isinstance(hashed, BaseException):
                         raise hashed
+                    self.logger.update_stats('suffix.hashes', hashed)
                     local_hash = recalc_hash
                     suffixes = [suffix for suffix in local_hash if
                             local_hash[suffix] != remote_hash.get(suffix, -1)]
@@ -442,6 +444,7 @@ class ObjectReplicator(Daemon):
                             headers={'Content-Length': '0'})
                         conn.getresponse().read()
                     self.suffix_sync += len(suffixes)
+                    self.logger.update_stats('suffix.syncs', len(suffixes))
                 except (Exception, Timeout):
                     self.logger.exception(_("Error syncing with node: %s") %
                                             node)
@@ -450,6 +453,7 @@ class ObjectReplicator(Daemon):
             self.logger.exception(_("Error syncing partition"))
         finally:
             self.partition_times.append(time.time() - begin)
+            self.logger.timing_since('partition.update.timing', begin)
 
     def stats_line(self):
         """
@@ -533,12 +537,14 @@ class ObjectReplicator(Daemon):
                 continue
             for partition in os.listdir(obj_path):
                 try:
-                    nodes = [node for node in
+                    part_nodes = \
                         self.object_ring.get_part_nodes(int(partition))
+                    nodes = [node for node in part_nodes
                              if node['id'] != local_dev['id']]
                     jobs.append(dict(path=join(obj_path, partition),
+                        device=local_dev['device'],
                         nodes=nodes,
-                        delete=len(nodes) > self.object_ring.replica_count - 1,
+                        delete=len(nodes) > len(part_nodes) - 1,
                         partition=partition))
                 except ValueError:
                     continue
@@ -589,12 +595,8 @@ class ObjectReplicator(Daemon):
         total = (time.time() - start) / 60
         self.logger.info(
             _("Object replication complete. (%.02f minutes)"), total)
-        if self.recon_enable:
-            try:
-                dump_recon_cache('object_replication_time', total, \
-                    self.recon_object)
-            except (Exception, Timeout):
-                self.logger.exception(_('Exception dumping recon cache'))
+        dump_recon_cache({'object_replication_time': total},
+                         self.rcache, self.logger)
 
     def run_forever(self, *args, **kwargs):
         self.logger.info(_("Starting object replicator in daemon mode."))
@@ -607,12 +609,8 @@ class ObjectReplicator(Daemon):
             total = (time.time() - start) / 60
             self.logger.info(
                 _("Object replication complete. (%.02f minutes)"), total)
-            if self.recon_enable:
-                try:
-                    dump_recon_cache('object_replication_time', total, \
-                        self.recon_object)
-                except (Exception, Timeout):
-                    self.logger.exception(_('Exception dumping recon cache'))
+            dump_recon_cache({'object_replication_time': total},
+                             self.rcache, self.logger)
             self.logger.debug(_('Replication sleeping for %s seconds.'),
                 self.run_pause)
             sleep(self.run_pause)
