@@ -53,7 +53,7 @@ from webob import Request, Response
 
 from swift.common.ring import Ring
 from swift.common.utils import cache_from_env, ContextPool, get_logger, \
-    get_remote_client, normalize_timestamp, split_path, TRUE_VALUES
+    get_remote_client, normalize_timestamp, split_path, TRUE_VALUES, public
 from swift.common.bufferedhttp import http_connect
 from swift.common.constraints import check_metadata, check_object_creation, \
     check_utf8, CONTAINER_LISTING_LIMIT, MAX_ACCOUNT_NAME_LENGTH, \
@@ -86,21 +86,6 @@ def update_headers(response, headers):
             response.headers[name] = value
 
 
-def public(func):
-    """
-    Decorator to declare which methods are publicly accessible as HTTP
-    requests
-
-    :param func: function to make public
-    """
-    func.publicly_accessible = True
-
-    @functools.wraps(func)
-    def wrapped(*a, **kw):
-        return func(*a, **kw)
-    return wrapped
-
-
 def delay_denial(func):
     """
     Decorator to declare which methods should have any swift.authorize call
@@ -178,9 +163,10 @@ class SegmentedIterable(object):
             if self.seek:
                 req.range = 'bytes=%s-' % self.seek
                 self.seek = 0
-            if self.segment > 10:
+            if self.segment > self.controller.app.rate_limit_after_segment:
                 sleep(max(self.next_get_time - time.time(), 0))
-                self.next_get_time = time.time() + 1
+                self.next_get_time = time.time() + \
+                    1.0 / self.controller.app.rate_limit_segments_per_sec
             shuffle(nodes)
             resp = self.controller.GETorHEAD_base(req, _('Object'), partition,
                 self.controller.iter_nodes(partition, nodes,
@@ -1932,6 +1918,10 @@ class BaseApplication(object):
             if a.strip()]
         self.deny_host_headers = [host.strip() for host in
             conf.get('deny_host_headers', '').split(',') if host.strip()]
+        self.rate_limit_after_segment = \
+            int(conf.get('rate_limit_after_segment', 10))
+        self.rate_limit_segments_per_sec = \
+            int(conf.get('rate_limit_segments_per_sec', 1))
 
     def get_controller(self, path):
         """
@@ -2022,11 +2012,8 @@ class BaseApplication(object):
             self.logger.client_ip = get_remote_client(req)
             try:
                 handler = getattr(controller, req.method)
-                if not getattr(handler, 'publicly_accessible'):
-                    handler = None
+                getattr(handler, 'publicly_accessible')
             except AttributeError:
-                handler = None
-            if not handler:
                 self.logger.increment('method_not_allowed')
                 return HTTPMethodNotAllowed(request=req)
             if path_parts['version']:
