@@ -93,6 +93,15 @@ class FakeApp(object):
     </body>
 </html>
             ''')(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c3b':
+            return self.listing(env, start_response,
+                                {'x-container-read': '.r:*',
+                                 'x-container-meta-web-index': 'index.html',
+                                 'x-container-meta-web-listings': 't'})
+        elif env['PATH_INFO'] == '/v1/a/c3b/index.html':
+            resp = Response(status='204 No Content')
+            resp.app_iter = iter([])
+            return resp(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c3/subdir':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c3/subdir/':
@@ -255,7 +264,7 @@ class FakeApp(object):
                   "last_modified":"2011-03-24T04:27:52.709100"}]
             '''.strip()
         elif 'prefix=' in env['QUERY_STRING']:
-            return Response(status='204 No Content')
+            return Response(status='204 No Content')(env, start_response)
         elif 'format=json' in env['QUERY_STRING']:
             headers.update({'X-Container-Object-Count': '11',
                             'X-Container-Bytes-Used': '73741',
@@ -355,6 +364,16 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank('/v1/a/c1').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 401)
 
+    def test_container1_web_mode_explicitly_off(self):
+        resp = Request.blank('/v1/a/c1',
+            headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 401)
+
+    def test_container1_web_mode_explicitly_on(self):
+        resp = Request.blank('/v1/a/c1',
+            headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
     def test_container2(self):
         resp = Request.blank('/v1/a/c2').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 200)
@@ -362,6 +381,19 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(len(resp.body.split('\n')),
                           int(resp.headers['x-container-object-count']))
 
+    def test_container2_web_mode_explicitly_off(self):
+        resp = Request.blank('/v1/a/c2',
+            headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(resp.content_type, 'text/plain')
+        self.assertEquals(len(resp.body.split('\n')),
+                          int(resp.headers['x-container-object-count']))
+
+    def test_container2_web_mode_explicitly_on(self):
+        resp = Request.blank('/v1/a/c2',
+            headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
     def test_container2onetxt(self):
         resp = Request.blank(
                 '/v1/a/c2/one.txt').get_response(self.test_staticweb)
@@ -375,6 +407,19 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(len(json.loads(resp.body)),
                           int(resp.headers['x-container-object-count']))
 
+    def test_container2json_web_mode_explicitly_off(self):
+        resp = Request.blank('/v1/a/c2?format=json',
+            headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(resp.content_type, 'application/json')
+        self.assertEquals(len(json.loads(resp.body)),
+                          int(resp.headers['x-container-object-count']))
+
+    def test_container2json_web_mode_explicitly_on(self):
+        resp = Request.blank('/v1/a/c2?format=json',
+            headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
     def test_container3(self):
         resp = Request.blank('/v1/a/c3').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 301)
@@ -432,6 +477,11 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(resp.status_int, 404)
         self.assert_("Chrome's 404 fancy-page sucks." not in resp.body)
 
+    def test_container3bindexhtml(self):
+        resp = Request.blank('/v1/a/c3b/').get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.body, '')
+
     def test_container4indexhtml(self):
         resp = Request.blank('/v1/a/c4/').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 200)
