@@ -17,14 +17,17 @@
 
 from __future__ import with_statement
 from test.unit import temptree
+import errno
 import logging
 import mimetools
 import os
-import errno
+import re
 import socket
 import sys
 import time
 import unittest
+from threading import Thread
+from Queue import Queue, Empty
 from getpass import getuser
 from shutil import rmtree
 from StringIO import StringIO
@@ -326,6 +329,8 @@ class TestUtils(unittest.TestCase):
                           'test1\ntest3\ntest4\n')
         # make sure notice lvl logs by default
         logger.notice('test6')
+        self.assertEquals(sio.getvalue(),
+                          'test1\ntest3\ntest4\ntest6\n')
 
     def test_clean_logger_exception(self):
         # setup stream logging
@@ -855,6 +860,274 @@ log_name = %(yarr)s'''
         self.assertFalse(utils.streq_const_time('a', 'aaaaa'))
         self.assertFalse(utils.streq_const_time('ABC123', 'abc123'))
 
+    def test_rsync_ip_ipv4_localhost(self):
+        self.assertEqual(utils.rsync_ip('127.0.0.1'), '127.0.0.1')
+
+    def test_rsync_ip_ipv6_random_ip(self):
+        self.assertEqual(
+            utils.rsync_ip('fe80:0000:0000:0000:0202:b3ff:fe1e:8329'),
+            '[fe80:0000:0000:0000:0202:b3ff:fe1e:8329]')
+
+    def test_rsync_ip_ipv6_ipv4_compatible(self):
+        self.assertEqual(
+            utils.rsync_ip('::ffff:192.0.2.128'), '[::ffff:192.0.2.128]')
+
+
+class TestStatsdLogging(unittest.TestCase):
+    def test_get_logger_statsd_client_not_specified(self):
+        logger = utils.get_logger({}, 'some-name', log_route='some-route')
+        # white-box construction validation
+        self.assertEqual(None, logger.logger.statsd_client)
+
+    def test_get_logger_statsd_client_defaults(self):
+        logger = utils.get_logger({'log_statsd_host': 'some.host.com'},
+                                  'some-name', log_route='some-route')
+        # white-box construction validation
+        self.assert_(isinstance(logger.logger.statsd_client, utils.StatsdClient))
+        self.assertEqual(logger.logger.statsd_client._host, 'some.host.com')
+        self.assertEqual(logger.logger.statsd_client._port, 8125)
+        self.assertEqual(logger.logger.statsd_client._prefix, 'some-name.')
+        self.assertEqual(logger.logger.statsd_client._default_sample_rate, 1)
+
+        logger.set_statsd_prefix('some-name.more-specific')
+        self.assertEqual(logger.logger.statsd_client._prefix,
+                         'some-name.more-specific.')
+        logger.set_statsd_prefix('')
+        self.assertEqual(logger.logger.statsd_client._prefix, '')
+
+    def test_get_logger_statsd_client_non_defaults(self):
+        logger = utils.get_logger({
+            'log_statsd_host': 'another.host.com',
+            'log_statsd_port': 9876,
+            'log_statsd_default_sample_rate': 0.75,
+            'log_statsd_metric_prefix': 'tomato.sauce',
+        }, 'some-name', log_route='some-route')
+        self.assertEqual(logger.logger.statsd_client._prefix,
+                         'tomato.sauce.some-name.')
+        logger.set_statsd_prefix('some-name.more-specific')
+        self.assertEqual(logger.logger.statsd_client._prefix,
+                         'tomato.sauce.some-name.more-specific.')
+        logger.set_statsd_prefix('')
+        self.assertEqual(logger.logger.statsd_client._prefix, 'tomato.sauce.')
+        self.assertEqual(logger.logger.statsd_client._host, 'another.host.com')
+        self.assertEqual(logger.logger.statsd_client._port, 9876)
+        self.assertEqual(logger.logger.statsd_client._default_sample_rate, 0.75)
+
+
+class TestStatsdLoggingDelegation(unittest.TestCase):
+    def setUp(self):
+        self.port = 9177
+        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        self.sock.bind(('localhost', self.port))
+        self.queue = Queue()
+        self.reader_thread = Thread(target=self.statsd_reader)
+        self.reader_thread.setDaemon(1)
+        self.reader_thread.start()
+
+    def tearDown(self):
+        # The "no-op when disabled" test doesn't set up a real logger, so
+        # create one here so we can tell the reader thread to stop.
+        if not getattr(self, 'logger', None):
+            self.logger = utils.get_logger({
+                'log_statsd_host': 'localhost',
+                'log_statsd_port': str(self.port),
+            }, 'some-name')
+        self.logger.increment('STOP')
+        self.reader_thread.join(timeout=4)
+        self.sock.close()
+        del self.logger
+        time.sleep(0.15)  # avoid occasional "Address already in use"?
+
+    def statsd_reader(self):
+        while True:
+            try:
+                payload = self.sock.recv(4096)
+                if payload and 'STOP' in payload:
+                    return 42
+                self.queue.put(payload)
+            except Exception, e:
+                sys.stderr.write('statsd_reader thread: %r' % (e,))
+                break
+
+    def _send_and_get(self, sender_fn, *args, **kwargs):
+        """
+        Because the client library may not actually send a packet with
+        sample_rate < 1, we keep trying until we get one through.
+        """
+        got = None
+        while not got:
+            sender_fn(*args, **kwargs)
+            try:
+                got = self.queue.get(timeout=0.5)
+            except Empty:
+                pass
+        return got
+
+    def assertStat(self, expected, sender_fn, *args, **kwargs):
+        got = self._send_and_get(sender_fn, *args, **kwargs)
+        return self.assertEqual(expected, got)
+
+    def assertStatMatches(self, expected_regexp, sender_fn, *args, **kwargs):
+        got = self._send_and_get(sender_fn, *args, **kwargs)
+        return self.assert_(re.search(expected_regexp, got),
+                            [got, expected_regexp])
+
+    def test_methods_are_no_ops_when_not_enabled(self):
+        logger = utils.get_logger({
+            # No "log_statsd_host" means "disabled"
+            'log_statsd_port': str(self.port),
+        }, 'some-name')
+        # Delegate methods are no-ops
+        self.assertEqual(None, logger.update_stats('foo', 88))
+        self.assertEqual(None, logger.update_stats('foo', 88, 0.57))
+        self.assertEqual(None, logger.update_stats('foo', 88, sample_rate=0.61))
+        self.assertEqual(None, logger.increment('foo'))
+        self.assertEqual(None, logger.increment('foo', 0.57))
+        self.assertEqual(None, logger.increment('foo', sample_rate=0.61))
+        self.assertEqual(None, logger.decrement('foo'))
+        self.assertEqual(None, logger.decrement('foo', 0.57))
+        self.assertEqual(None, logger.decrement('foo', sample_rate=0.61))
+        self.assertEqual(None, logger.timing('foo', 88.048))
+        self.assertEqual(None, logger.timing('foo', 88.57, 0.34))
+        self.assertEqual(None, logger.timing('foo', 88.998, sample_rate=0.82))
+        self.assertEqual(None, logger.timing_since('foo', 8938))
+        self.assertEqual(None, logger.timing_since('foo', 8948, 0.57))
+        self.assertEqual(None, logger.timing_since('foo', 849398,
+                                                   sample_rate=0.61))
+        # Now, the queue should be empty (no UDP packets sent)
+        self.assertRaises(Empty, self.queue.get_nowait)
+
+    def test_delegate_methods_with_no_default_sample_rate(self):
+        self.logger = utils.get_logger({
+            'log_statsd_host': 'localhost',
+            'log_statsd_port': str(self.port),
+        }, 'some-name')
+        self.assertStat('some-name.some.counter:1|c', self.logger.increment,
+                        'some.counter')
+        self.assertStat('some-name.some.counter:-1|c', self.logger.decrement,
+                        'some.counter')
+        self.assertStat('some-name.some.operation:4900.0|ms',
+                        self.logger.timing, 'some.operation', 4.9 * 1000)
+        self.assertStatMatches('some-name\.another\.operation:\d+\.\d+\|ms',
+                               self.logger.timing_since, 'another.operation',
+                               time.time())
+        self.assertStat('some-name.another.counter:42|c',
+                        self.logger.update_stats, 'another.counter', 42)
+
+        # Each call can override the sample_rate (also, bonus prefix test)
+        self.logger.set_statsd_prefix('pfx')
+        self.assertStat('pfx.some.counter:1|c|@0.972', self.logger.increment,
+                        'some.counter', sample_rate=0.972)
+        self.assertStat('pfx.some.counter:-1|c|@0.972', self.logger.decrement,
+                        'some.counter', sample_rate=0.972)
+        self.assertStat('pfx.some.operation:4900.0|ms|@0.972',
+                        self.logger.timing, 'some.operation', 4.9 * 1000,
+                        sample_rate=0.972)
+        self.assertStatMatches('pfx\.another\.op:\d+\.\d+\|ms|@0.972',
+                               self.logger.timing_since, 'another.op',
+                               time.time(), sample_rate=0.972)
+        self.assertStat('pfx.another.counter:3|c|@0.972',
+                        self.logger.update_stats, 'another.counter', 3,
+                        sample_rate=0.972)
+
+        # Can override sample_rate with non-keyword arg
+        self.logger.set_statsd_prefix('')
+        self.assertStat('some.counter:1|c|@0.939', self.logger.increment,
+                        'some.counter', 0.939)
+        self.assertStat('some.counter:-1|c|@0.939', self.logger.decrement,
+                        'some.counter', 0.939)
+        self.assertStat('some.operation:4900.0|ms|@0.939',
+                        self.logger.timing, 'some.operation',
+                        4.9 * 1000, 0.939)
+        self.assertStatMatches('another\.op:\d+\.\d+\|ms|@0.939',
+                               self.logger.timing_since, 'another.op',
+                               time.time(), 0.939)
+        self.assertStat('another.counter:3|c|@0.939',
+                        self.logger.update_stats, 'another.counter', 3, 0.939)
+
+    def test_delegate_methods_with_default_sample_rate(self):
+        self.logger = utils.get_logger({
+            'log_statsd_host': 'localhost',
+            'log_statsd_port': str(self.port),
+            'log_statsd_default_sample_rate': '0.93',
+        }, 'pfx')
+        self.assertStat('pfx.some.counter:1|c|@0.93', self.logger.increment,
+                        'some.counter')
+        self.assertStat('pfx.some.counter:-1|c|@0.93', self.logger.decrement,
+                        'some.counter')
+        self.assertStat('pfx.some.operation:4760.0|ms|@0.93',
+                        self.logger.timing, 'some.operation', 4.76 * 1000)
+        self.assertStatMatches('pfx\.another\.op:\d+\.\d+\|ms|@0.93',
+                               self.logger.timing_since, 'another.op',
+                               time.time())
+        self.assertStat('pfx.another.counter:3|c|@0.93',
+                        self.logger.update_stats, 'another.counter', 3)
+
+        # Each call can override the sample_rate
+        self.assertStat('pfx.some.counter:1|c|@0.9912', self.logger.increment,
+                        'some.counter', sample_rate=0.9912)
+        self.assertStat('pfx.some.counter:-1|c|@0.9912', self.logger.decrement,
+                        'some.counter', sample_rate=0.9912)
+        self.assertStat('pfx.some.operation:4900.0|ms|@0.9912',
+                        self.logger.timing, 'some.operation', 4.9 * 1000,
+                        sample_rate=0.9912)
+        self.assertStatMatches('pfx\.another\.op:\d+\.\d+\|ms|@0.9912',
+                               self.logger.timing_since, 'another.op',
+                               time.time(), sample_rate=0.9912)
+        self.assertStat('pfx.another.counter:3|c|@0.9912',
+                        self.logger.update_stats, 'another.counter', 3,
+                        sample_rate=0.9912)
+
+        # Can override sample_rate with non-keyword arg
+        self.logger.set_statsd_prefix('')
+        self.assertStat('some.counter:1|c|@0.987654', self.logger.increment,
+                        'some.counter', 0.987654)
+        self.assertStat('some.counter:-1|c|@0.987654', self.logger.decrement,
+                        'some.counter', 0.987654)
+        self.assertStat('some.operation:4900.0|ms|@0.987654',
+                        self.logger.timing, 'some.operation',
+                        4.9 * 1000, 0.987654)
+        self.assertStatMatches('another\.op:\d+\.\d+\|ms|@0.987654',
+                               self.logger.timing_since, 'another.op',
+                               time.time(), 0.987654)
+        self.assertStat('another.counter:3|c|@0.987654',
+                        self.logger.update_stats, 'another.counter',
+                        3, 0.987654)
+
+    def test_delegate_methods_with_metric_prefix(self):
+        self.logger = utils.get_logger({
+            'log_statsd_host': 'localhost',
+            'log_statsd_port': str(self.port),
+            'log_statsd_metric_prefix': 'alpha.beta',
+        }, 'pfx')
+        self.assertStat('alpha.beta.pfx.some.counter:1|c',
+                        self.logger.increment, 'some.counter')
+        self.assertStat('alpha.beta.pfx.some.counter:-1|c',
+                        self.logger.decrement, 'some.counter')
+        self.assertStat('alpha.beta.pfx.some.operation:4760.0|ms',
+                        self.logger.timing, 'some.operation', 4.76 * 1000)
+        self.assertStatMatches(
+            'alpha\.beta\.pfx\.another\.op:\d+\.\d+\|ms',
+            self.logger.timing_since, 'another.op', time.time())
+        self.assertStat('alpha.beta.pfx.another.counter:3|c',
+                        self.logger.update_stats, 'another.counter', 3)
+
+        self.logger.set_statsd_prefix('')
+        self.assertStat('alpha.beta.some.counter:1|c|@0.9912',
+                        self.logger.increment, 'some.counter',
+                        sample_rate=0.9912)
+        self.assertStat('alpha.beta.some.counter:-1|c|@0.9912',
+                        self.logger.decrement, 'some.counter', 0.9912)
+        self.assertStat('alpha.beta.some.operation:4900.0|ms|@0.9912',
+                        self.logger.timing, 'some.operation', 4.9 * 1000,
+                        sample_rate=0.9912)
+        self.assertStatMatches('alpha\.beta\.another\.op:\d+\.\d+\|ms|@0.9912',
+                               self.logger.timing_since, 'another.op',
+                               time.time(), sample_rate=0.9912)
+        self.assertStat('alpha.beta.another.counter:3|c|@0.9912',
+                        self.logger.update_stats, 'another.counter', 3,
+                        sample_rate=0.9912)
+
 
 if __name__ == '__main__':
     unittest.main()
