@@ -15,6 +15,8 @@
 
 import unittest
 
+import re
+from test.unit import FakeLogger
 from swift.container import sync
 from swift.common import utils
 from swift.common.client import ClientException
@@ -765,18 +767,6 @@ class TestContainerSync(unittest.TestCase):
                                 contents=None, proxy=None):
                 raise ClientException('test client exception', http_status=401)
 
-            class FakeLogger(object):
-
-                def __init__(self):
-                    self.err = ''
-                    self.exc = ''
-
-                def info(self, err, *args, **kwargs):
-                    self.err = err
-
-                def exception(self, exc, *args, **kwargs):
-                    self.exc = exc
-
             sync.direct_get_object = fake_direct_get_object
             sync.put_object = fake_put_object
             cs.logger = FakeLogger()
@@ -786,7 +776,8 @@ class TestContainerSync(unittest.TestCase):
                 'key', FakeContainerBroker('broker'), {'account': 'a',
                 'container': 'c'}))
             self.assertEquals(cs.container_puts, 2)
-            self.assertTrue(cs.logger.err.startswith('Unauth '))
+            self.assert_(re.match('Unauth ',
+                                  cs.logger.log_dict['info'][0][0][0]))
 
             def fake_put_object(sync_to, name=None, headers=None,
                                 contents=None, proxy=None):
@@ -794,12 +785,14 @@ class TestContainerSync(unittest.TestCase):
 
             sync.put_object = fake_put_object
             # Fail due to 404
+            cs.logger = FakeLogger()
             self.assertFalse(cs.container_sync_row({'deleted': False,
                 'name': 'object', 'created_at': '1.2'}, 'http://sync/to/path',
                 'key', FakeContainerBroker('broker'), {'account': 'a',
                 'container': 'c'}))
             self.assertEquals(cs.container_puts, 2)
-            self.assertTrue(cs.logger.err.startswith('Not found '))
+            self.assert_(re.match('Not found ',
+                                  cs.logger.log_dict['info'][0][0][0]))
 
             def fake_put_object(sync_to, name=None, headers=None,
                                 contents=None, proxy=None):
@@ -812,7 +805,9 @@ class TestContainerSync(unittest.TestCase):
                 'key', FakeContainerBroker('broker'), {'account': 'a',
                 'container': 'c'}))
             self.assertEquals(cs.container_puts, 2)
-            self.assertTrue(cs.logger.exc.startswith('ERROR Syncing '))
+            self.assertTrue(
+                cs.logger.log_dict['exception'][0][0][0].startswith(
+                    'ERROR Syncing '))
         finally:
             sync.shuffle = orig_shuffle
             sync.put_object = orig_put_object
